@@ -1,10 +1,12 @@
 """Provide DynamoDB client."""
+
 import atexit
 import logging
 import sys
+from collections.abc import Generator
 from os import environ
 from pathlib import Path
-from typing import Any, Dict, Generator, List, Optional, Set, Union
+from typing import Any
 
 import boto3
 import click
@@ -31,7 +33,7 @@ _logger = logging.getLogger(__name__)
 class DynamoDbDatabase(AbstractDatabase):
     """Database class employing DynamoDB."""
 
-    def __init__(self, db_url: Optional[str] = None, **db_args) -> None:
+    def __init__(self, db_url: str | None = None, **db_args) -> None:
         """Initialize Database class.
 
         :param str db_url: URL endpoint for DynamoDB source
@@ -44,20 +46,18 @@ class DynamoDbDatabase(AbstractDatabase):
 
         if AWS_ENV_VAR_NAME in environ:
             if "GENE_TEST" in environ:
-                raise DatabaseInitializationException(
-                    f"Cannot have both GENE_TEST and {AWS_ENV_VAR_NAME} set."
-                )  # noqa: E501
+                err_msg = f"Cannot have both GENE_TEST and {AWS_ENV_VAR_NAME} set."
+                raise DatabaseInitializationException(err_msg)
 
             aws_env = environ[AWS_ENV_VAR_NAME]
             if aws_env not in VALID_AWS_ENV_NAMES:
-                raise DatabaseInitializationException(
-                    f"{AWS_ENV_VAR_NAME} must be one of {VALID_AWS_ENV_NAMES}"
-                )  # noqa: E501
+                err_msg = f"{AWS_ENV_VAR_NAME} must be one of {VALID_AWS_ENV_NAMES}"
+                raise DatabaseInitializationException(err_msg)
 
             skip_confirmation = environ.get(SKIP_AWS_DB_ENV_NAME)
             if (not skip_confirmation) or (
                 skip_confirmation and skip_confirmation != "true"
-            ):  # noqa: E501
+            ):
                 confirm_aws_db_use(environ[AWS_ENV_VAR_NAME])
 
             boto_params = {"region_name": region_name}
@@ -89,7 +89,7 @@ class DynamoDbDatabase(AbstractDatabase):
         self._cached_sources = {}
         atexit.register(self.close_connection)
 
-    def list_tables(self) -> List[str]:
+    def list_tables(self) -> list[str]:
         """Return names of tables in database.
 
         :return: Table names in DynamoDB
@@ -156,7 +156,7 @@ class DynamoDbDatabase(AbstractDatabase):
         existing_tables = self.list_tables()
         exists = self.gene_table in existing_tables
         if not exists:
-            _logger.info(f"{self.gene_table} table is missing or unavailable.")
+            _logger.info("%s table is missing or unavailable.", self.gene_table)
         return exists
 
     def check_tables_populated(self) -> bool:
@@ -201,7 +201,7 @@ class DynamoDbDatabase(AbstractDatabase):
         if not self.check_schema_initialized():
             self._create_genes_table()
 
-    def get_source_metadata(self, src_name: Union[str, SourceName]) -> Dict:
+    def get_source_metadata(self, src_name: str | SourceName) -> dict:
         """Get license, versioning, data lookup, etc information for a source.
 
         :param src_name: name of the source to get data for
@@ -210,22 +210,21 @@ class DynamoDbDatabase(AbstractDatabase):
             src_name = src_name.value
         if src_name in self._cached_sources:
             return self._cached_sources[src_name]
-        else:
-            pk = f"{src_name.lower()}##source"
-            concept_id = f"source:{src_name.lower()}"
-            metadata = self.genes.get_item(
-                Key={"label_and_type": pk, "concept_id": concept_id}
-            ).get("Item")
-            if not metadata:
-                raise DatabaseReadException(
-                    f"Unable to retrieve data for source {src_name}"
-                )
-            self._cached_sources[src_name] = metadata
-            return metadata
+
+        pk = f"{src_name.lower()}##source"
+        concept_id = f"source:{src_name.lower()}"
+        metadata = self.genes.get_item(
+            Key={"label_and_type": pk, "concept_id": concept_id}
+        ).get("Item")
+        if not metadata:
+            err_msg = f"Unable to retrieve data for source {src_name}"
+            raise DatabaseReadException(err_msg)
+        self._cached_sources[src_name] = metadata
+        return metadata
 
     def get_record_by_id(
         self, concept_id: str, case_sensitive: bool = True, merge: bool = False
-    ) -> Optional[Dict]:
+    ) -> dict | None:
         """Fetch record corresponding to provided concept ID
 
         :param str concept_id: concept ID for gene record
@@ -246,23 +245,23 @@ class DynamoDbDatabase(AbstractDatabase):
                     Key={"label_and_type": pk, "concept_id": concept_id}
                 )
                 return match["Item"]
-            else:
-                exp = Key("label_and_type").eq(pk)
-                response = self.genes.query(KeyConditionExpression=exp)
-                record = response["Items"][0]
-                del record["label_and_type"]
-                return record
+
+            exp = Key("label_and_type").eq(pk)
+            response = self.genes.query(KeyConditionExpression=exp)
+            record = response["Items"][0]
+            del record["label_and_type"]
+            return record
         except ClientError as e:
             _logger.error(
-                f"boto3 client error on get_records_by_id for "
-                f"search term {concept_id}: "
-                f"{e.response['Error']['Message']}"
+                "boto3 client error on get_records_by_id for search term %s: %s",
+                concept_id,
+                e.response["Error"]["Message"],
             )
             return None
         except (KeyError, IndexError):  # record doesn't exist
             return None
 
-    def get_refs_by_type(self, search_term: str, ref_type: RefType) -> List[str]:
+    def get_refs_by_type(self, search_term: str, ref_type: RefType) -> list[str]:
         """Retrieve concept IDs for records matching the user's query. Other methods
         are responsible for actually retrieving full records.
 
@@ -277,13 +276,13 @@ class DynamoDbDatabase(AbstractDatabase):
             return [m["concept_id"] for m in matches.get("Items", None)]
         except ClientError as e:
             _logger.error(
-                f"boto3 client error on get_refs_by_type for "
-                f"search term {search_term}: "
-                f"{e.response['Error']['Message']}"
+                "boto3 client error on get_refs_by_type for search term %s: %s",
+                search_term,
+                e.response["Error"]["Message"],
             )
             return []
 
-    def get_all_concept_ids(self) -> Set[str]:
+    def get_all_concept_ids(self) -> set[str]:
         """Retrieve concept IDs for use in generating normalized records.
 
         :return: List of concept IDs as strings.
@@ -301,14 +300,13 @@ class DynamoDbDatabase(AbstractDatabase):
             else:
                 response = self.genes.scan(**params)
             records = response["Items"]
-            for record in records:
-                concept_ids.append(record["concept_id"])
+            concept_ids.extend(record["concept_id"] for record in records)
             last_evaluated_key = response.get("LastEvaluatedKey")
             if not last_evaluated_key:
                 break
         return set(concept_ids)
 
-    def get_all_records(self, record_type: RecordType) -> Generator[Dict, None, None]:
+    def get_all_records(self, record_type: RecordType) -> Generator[dict, None, None]:
         """Retrieve all source or normalized records. Either return all source records,
         or all records that qualify as "normalized" (i.e., merged groups + source
         records that are otherwise ungrouped).
@@ -341,7 +339,7 @@ class DynamoDbDatabase(AbstractDatabase):
                 else:
                     if (
                         incoming_record_type == RecordType.IDENTITY
-                        and not record.get("merge_ref")  # noqa: E501
+                        and not record.get("merge_ref")
                     ) or incoming_record_type == RecordType.MERGER:
                         yield record
             last_evaluated_key = response.get("LastEvaluatedKey")
@@ -364,9 +362,9 @@ class DynamoDbDatabase(AbstractDatabase):
         try:
             self.genes.put_item(Item=metadata_item)
         except ClientError as e:
-            raise DatabaseWriteException(e)
+            raise DatabaseWriteException(e) from e
 
-    def add_record(self, record: Dict, src_name: SourceName) -> None:
+    def add_record(self, record: dict, src_name: SourceName) -> None:
         """Add new record to database.
 
         :param Dict record: record to upload
@@ -381,8 +379,9 @@ class DynamoDbDatabase(AbstractDatabase):
             self.batch.put_item(Item=record)
         except ClientError as e:
             _logger.error(
-                "boto3 client error on add_record for "
-                f"{concept_id}: {e.response['Error']['Message']}"
+                "boto3 client error on add_record for %s: %s",
+                concept_id,
+                e.response["Error"]["Message"],
             )
         for attr_type, item_type in ITEM_TYPES.items():
             if attr_type in record:
@@ -398,7 +397,7 @@ class DynamoDbDatabase(AbstractDatabase):
                         item, record["concept_id"], item_type, src_name
                     )
 
-    def add_merged_record(self, record: Dict) -> None:
+    def add_merged_record(self, record: dict) -> None:
         """Add merged record to database.
 
         :param record: merged record to add
@@ -413,8 +412,9 @@ class DynamoDbDatabase(AbstractDatabase):
             self.batch.put_item(Item=record)
         except ClientError as e:
             _logger.error(
-                "boto3 client error on add_record for "
-                f"{concept_id}: {e.response['Error']['Message']}"
+                "boto3 client error on add_record for " "%s: %s",
+                concept_id,
+                e.response["Error"]["Message"],
             )
 
     def _add_ref_record(
@@ -439,9 +439,11 @@ class DynamoDbDatabase(AbstractDatabase):
             self.batch.put_item(Item=record)
         except ClientError as e:
             _logger.error(
-                f"boto3 client error adding reference {term} for "
-                f"{concept_id} with match type {ref_type}: "
-                f"{e.response['Error']['Message']}"
+                "boto3 client error adding reference %s for %s with match type %s: %s",
+                term,
+                concept_id,
+                ref_type,
+                e.response["Error"]["Message"],
             )
 
     def update_merge_ref(self, concept_id: str, merge_ref: Any) -> None:  # noqa: ANN401
@@ -466,14 +468,15 @@ class DynamoDbDatabase(AbstractDatabase):
         except ClientError as e:
             code = e.response.get("Error", {}).get("Code")
             if code == "ConditionalCheckFailedException":
-                raise DatabaseWriteException(
+                err_msg = (
                     f"No such record exists for keys {label_and_type}, {concept_id}"
                 )
-            else:
-                _logger.error(
-                    f"boto3 client error in `database.update_record()`: "
-                    f"{e.response['Error']['Message']}"
-                )
+                raise DatabaseWriteException(err_msg) from e
+
+            _logger.error(
+                "boto3 client error in `database.update_record()`: %s",
+                e.response["Error"]["Message"],
+            )
 
     def delete_normalized_concepts(self) -> None:
         """Remove merged records from the database. Use when performing a new update
@@ -495,7 +498,7 @@ class DynamoDbDatabase(AbstractDatabase):
                         ),
                     )
                 except ClientError as e:
-                    raise DatabaseReadException(e)
+                    raise DatabaseReadException(e) from e
                 records = response["Items"]
                 if not records:
                     break
@@ -522,23 +525,23 @@ class DynamoDbDatabase(AbstractDatabase):
                     KeyConditionExpression=Key("src_name").eq(src_name.value),
                 )
             except ClientError as e:
-                raise DatabaseReadException(e)
+                raise DatabaseReadException(e) from e
             records = response["Items"]
             if not records:
                 break
             with self.genes.batch_writer(
                 overwrite_by_pkeys=["label_and_type", "concept_id"]
             ) as batch:
-                for record in records:
-                    try:
+                try:
+                    for record in records:
                         batch.delete_item(
                             Key={
                                 "label_and_type": record["label_and_type"],
                                 "concept_id": record["concept_id"],
                             }
                         )
-                    except ClientError as e:
-                        raise DatabaseWriteException(e)
+                except ClientError as e:
+                    raise DatabaseWriteException(e) from e
 
     def complete_write_transaction(self) -> None:
         """Conclude transaction or batch writing if relevant."""
@@ -549,7 +552,7 @@ class DynamoDbDatabase(AbstractDatabase):
         """Perform any manual connection closure procedures if necessary."""
         self.batch.__exit__(*sys.exc_info())
 
-    def load_from_remote(self, url: Optional[str] = None) -> None:
+    def load_from_remote(self, url: str | None = None) -> None:
         """Load DB from remote dump. Not available for DynamoDB database backend.
 
         :param url: remote location to retrieve gzipped dump file from
