@@ -1,9 +1,9 @@
 """Defines ETL methods for the NCBI data source."""
+
 import csv
 import logging
 import re
 from pathlib import Path
-from typing import Dict, List, Optional
 
 import gffutils
 from wags_tails import NcbiGenomeData
@@ -18,6 +18,7 @@ from gene.etl.exceptions import (
 from gene.schemas import (
     Annotation,
     Chromosome,
+    DataLicenseAttributes,
     NamespacePrefix,
     SourceMeta,
     SourceName,
@@ -34,7 +35,7 @@ class NCBI(Base):
         self,
         database: AbstractDatabase,
         seqrepo_dir: Path = SEQREPO_ROOT_DIR,
-        data_path: Optional[Path] = None,
+        data_path: Path | None = None,
         silent: bool = True,
     ) -> None:
         """Instantiate Base class.
@@ -59,7 +60,7 @@ class NCBI(Base):
         gene_paths: NcbiGenePaths
         gene_paths, self._version = self._data_source.get_latest(
             from_local=use_existing
-        )  # type: ignore
+        )
         self._info_src = gene_paths.gene_info
         self._history_src = gene_paths.gene_history
         self._gene_url = (
@@ -68,13 +69,13 @@ class NCBI(Base):
         self._history_url = "ftp.ncbi.nlm.nih.govgene/DATA/gene_history.gz"
         self._assembly_url = "ftp.ncbi.nlm.nih.govgenomes/refseq/vertebrate_mammalian/Homo_sapiens/latest_assembly_versions/"
 
-    def _get_prev_symbols(self) -> Dict[str, str]:
+    def _get_prev_symbols(self) -> dict[str, str]:
         """Store a gene's symbol history.
 
         :return: A dictionary of a gene's previous symbols
         """
         # get symbol history
-        history_file = open(self._history_src, "r")
+        history_file = self._history_src.open()
         history = csv.reader(history_file, delimiter="\t")
         next(history)
         prev_symbols = {}
@@ -83,7 +84,7 @@ class NCBI(Base):
             if row[0] == "9606":
                 if row[1] != "-":
                     gene_id = row[1]
-                    if gene_id in prev_symbols.keys():
+                    if gene_id in prev_symbols:
                         prev_symbols[gene_id].append(row[3])
                     else:
                         prev_symbols[gene_id] = [row[3]]
@@ -98,7 +99,7 @@ class NCBI(Base):
         history_file.close()
         return prev_symbols
 
-    def _add_xrefs_associated_with(self, val: List[str], params: Dict) -> None:
+    def _add_xrefs_associated_with(self, val: list[str], params: dict) -> None:
         """Add xrefs and associated_with refs to a transformed gene.
 
         :param val: A list of source ids for a given gene
@@ -130,58 +131,56 @@ class NCBI(Base):
                 if prefix:
                     params["associated_with"].append(f"{prefix}:{src_id}")
                 else:
-                    _logger.info(f"{src_name} is not in NameSpacePrefix.")
+                    _logger.info("%s is not in NameSpacePrefix.", src_name)
         if not params["xrefs"]:
             del params["xrefs"]
         if not params["associated_with"]:
             del params["associated_with"]
 
-    def _get_gene_info(self, prev_symbols: Dict[str, str]) -> Dict[str, str]:
+    def _get_gene_info(self, prev_symbols: dict[str, str]) -> dict[str, str]:
         """Store genes from NCBI info file.
 
         :param prev_symbols: A dictionary of a gene's previous symbols
         :return: A dictionary of gene's from the NCBI info file.
         """
-        # open info file, skip headers
-        info_file = open(self._info_src, "r")
-        info = csv.reader(info_file, delimiter="\t")
-        next(info)
+        info_genes = {}
 
-        info_genes = dict()
-        for row in info:
-            params = dict()
-            params["concept_id"] = f"{NamespacePrefix.NCBI.value}:{row[1]}"
-            # get symbol
-            params["symbol"] = row[2]
-            # get aliases
-            if row[4] != "-":
-                params["aliases"] = row[4].split("|")
-            else:
-                params["aliases"] = []
-            # get associated_with
-            if row[5] != "-":
-                associated_with = row[5].split("|")
-                self._add_xrefs_associated_with(associated_with, params)
-            # get chromosome location
-            vrs_chr_location = self._get_vrs_chr_location(row, params)
-            if "exclude" in vrs_chr_location:
-                # Exclude genes with multiple distinct locations (e.g. OMS)
-                continue
-            if not vrs_chr_location:
-                vrs_chr_location = []
-            params["locations"] = vrs_chr_location
-            # get label
-            if row[8] != "-":
-                params["label"] = row[8]
-            # add prev symbols
-            if row[1] in prev_symbols.keys():
-                params["previous_symbols"] = prev_symbols[row[1]]
-            info_genes[params["symbol"]] = params
-            # get type
-            params["gene_type"] = row[9]
+        # open info file, skip headers
+        with self._info_src.open() as info_file:
+            info = csv.reader(info_file, delimiter="\t")
+            next(info)
+
+            for row in info:
+                params = {"locations": []}
+                params["concept_id"] = f"{NamespacePrefix.NCBI.value}:{row[1]}"
+                # get symbol
+                params["symbol"] = row[2]
+                # get aliases
+                if row[4] != "-":
+                    params["aliases"] = row[4].split("|")
+                else:
+                    params["aliases"] = []
+                # get associated_with
+                if row[5] != "-":
+                    associated_with = row[5].split("|")
+                    self._add_xrefs_associated_with(associated_with, params)
+                # get chromosome location
+                do_exclude = self._add_location_annotations(row, params)
+                if do_exclude:
+                    # Exclude genes with multiple distinct locations (e.g. OMS)
+                    continue
+                # get label
+                if row[8] != "-":
+                    params["label"] = row[8]
+                # add prev symbols
+                if row[1] in prev_symbols:
+                    params["previous_symbols"] = prev_symbols[row[1]]
+                info_genes[params["symbol"]] = params
+                # get type
+                params["gene_type"] = row[9]
         return info_genes
 
-    def _get_gene_gff(self, db: gffutils.FeatureDB, info_genes: Dict) -> None:
+    def _get_gene_gff(self, db: gffutils.FeatureDB, info_genes: dict) -> None:
         """Store genes from NCBI gff file.
 
         :param db: GFF database
@@ -197,7 +196,7 @@ class NCBI(Base):
                         params = info_genes.get(symbol)
                         vrs_sq_location = self._get_vrs_sq_location(db, params, f_id)
                         if vrs_sq_location:
-                            params["locations"].append(vrs_sq_location)  # type: ignore
+                            params["locations"].append(vrs_sq_location)
                     else:
                         # Need to add entire gene
                         gene = self._add_gff_gene(db, f, f_id)
@@ -205,7 +204,7 @@ class NCBI(Base):
 
     def _add_gff_gene(
         self, db: gffutils.FeatureDB, f: gffutils.Feature, f_id: str
-    ) -> Optional[Dict]:
+    ) -> dict | None:
         """Create a transformed gene recor from NCBI gff file.
 
         :param db: GFF database
@@ -213,18 +212,18 @@ class NCBI(Base):
         :param f_id: The feature's ID
         :return: A gene dictionary if the ID attribute exists. Else return None.
         """
-        params = dict()
+        params = {}
         params["src_name"] = SourceName.NCBI.value
         self._add_attributes(f, params)
         sq_loc = self._get_vrs_sq_location(db, params, f_id)
         if sq_loc:
             params["locations"] = [sq_loc]
         else:
-            params["locations"] = list()
+            params["locations"] = []
         params["label_and_type"] = f"{params['concept_id'].lower()}##identity"
         return params
 
-    def _add_attributes(self, f: gffutils.feature.Feature, gene: Dict) -> None:
+    def _add_attributes(self, f: gffutils.feature.Feature, gene: dict) -> None:
         """Add concept_id, symbol, and xrefs/associated_with to a gene record.
 
         :param gffutils.feature.Feature f: A gene from the data
@@ -246,8 +245,8 @@ class NCBI(Base):
                     gene["symbol"] = val
 
     def _get_vrs_sq_location(
-        self, db: gffutils.FeatureDB, params: Dict, f_id: str
-    ) -> Dict:
+        self, db: gffutils.FeatureDB, params: dict, f_id: str
+    ) -> dict:
         """Store GA4GH VRS SequenceLocation in a gene record.
         https://vr-spec.readthedocs.io/en/1.1/terms_and_model.html#sequencelocation
 
@@ -260,14 +259,14 @@ class NCBI(Base):
         params["strand"] = gene.strand
         return self._get_sequence_location(gene.seqid, gene, params)
 
-    def _get_xref_associated_with(self, src_name: str, src_id: str) -> Dict:
+    def _get_xref_associated_with(self, src_name: str, src_id: str) -> dict:
         """Get xref or associated_with ref.
 
         :param src_name: Source name
         :param src_id: The source's accession number
         :return: A dict containing an xref or associated_with ref
         """
-        source = dict()
+        source = {}
         if src_name.startswith("HGNC"):
             source["xrefs"] = [f"{NamespacePrefix.HGNC.value}:{src_id}"]
         elif src_name.startswith("NCBI"):
@@ -280,22 +279,21 @@ class NCBI(Base):
             source["associated_with"] = [f"{NamespacePrefix.RFAM.value}:{src_id}"]
         return source
 
-    def _get_vrs_chr_location(self, row: List[str], params: Dict) -> List:
-        """Store GA4GH VRS ChromosomeLocation in a gene record.
-        https://vr-spec.readthedocs.io/en/1.1/terms_and_model.html#chromosomelocation
+    def _add_location_annotations(self, row: list[str], params: dict) -> bool:
+        """Add location annotations to ``params``
 
         :param row: A row in NCBI data file
-        :param params: A transformed gene record
-        :return: A list of GA4GH VRS ChromosomeLocations
+        :param params: A transformed gene record. This may get mutated in place.
+        :return: ``True`` if gene found with multiple distinct locations.
+            ``False``, otherwise.
         """
-        params["location_annotations"] = list()
+        params["location_annotations"] = []
         chromosomes_locations = self._set_chromsomes_locations(row, params)
         locations = chromosomes_locations["locations"]
         chromosomes = chromosomes_locations["chromosomes"]
         if chromosomes_locations["exclude"]:
-            return ["exclude"]
+            return True
 
-        location_list = list()
         if chromosomes and not locations:
             for chromosome in chromosomes:
                 if chromosome == "MT":
@@ -303,12 +301,12 @@ class NCBI(Base):
                 else:
                     params["location_annotations"].append(chromosome.strip())
         elif locations:
-            self._add_chromosome_location(locations, location_list, params)
+            self._add_chromosome_location(locations, params)
         if not params["location_annotations"]:
             del params["location_annotations"]
-        return location_list
+        return False
 
-    def _set_chromsomes_locations(self, row: List[str], params: Dict) -> Dict:
+    def _set_chromsomes_locations(self, row: list[str], params: dict) -> dict:
         """Set chromosomes and locations for a given gene record.
 
         :param row: A gene row in the NCBI data file
@@ -317,18 +315,18 @@ class NCBI(Base):
         """
         chromosomes = None
         if row[6] != "-":
-            if "|" in row[6]:
-                chromosomes = row[6].split("|")
-            else:
-                chromosomes = [row[6]]
+            chromosomes = row[6].split("|") if "|" in row[6] else [row[6]]
 
-            if len(chromosomes) >= 2:
-                if chromosomes and "X" not in chromosomes and "Y" not in chromosomes:
-                    _logger.info(
-                        f"{row[2]} contains multiple distinct "
-                        f"chromosomes: {chromosomes}."
-                    )
-                    chromosomes = None
+            if (
+                len(chromosomes) >= 2
+                and chromosomes
+                and "X" not in chromosomes
+                and "Y" not in chromosomes
+            ):
+                _logger.info(
+                    "%s contains multiple distinct chromosomes: %s", row[2], chromosomes
+                )
+                chromosomes = None
 
         locations = None
         exclude = False
@@ -343,15 +341,14 @@ class NCBI(Base):
                 locations = [row[7]]
 
             # Sometimes locations will store the same location twice
-            if len(locations) == 2:
-                if locations[0] == locations[1]:
-                    locations = [locations[0]]
+            if len(locations) == 2 and locations[0] == locations[1]:
+                locations = [locations[0]]
 
             # Exclude genes where there are multiple distinct locations
             # i.e. OMS: '10q26.3', '19q13.42-q13.43', '3p25.3'
             if len(locations) > 2:
                 _logger.info(
-                    f"{row[2]} contains multiple distinct " f"locations: {locations}."
+                    "%s contains multiple distinct locations: %s", row[2], locations
                 )
                 locations = None
                 exclude = True
@@ -362,24 +359,21 @@ class NCBI(Base):
                     loc = locations[i].strip()
                     if not re.match("^([1-9][0-9]?|X[pq]?|Y[pq]?)", loc):
                         _logger.info(
-                            f"{row[2]} contains invalid map location:" f"{loc}."
+                            "%s contains invalid map location: %s", row[2], loc
                         )
                         params["location_annotations"].append(loc)
                         del locations[i]
         return {"locations": locations, "chromosomes": chromosomes, "exclude": exclude}
 
-    def _add_chromosome_location(
-        self, locations: List, location_list: List, params: Dict
-    ) -> None:
+    def _add_chromosome_location(self, locations: list, params: dict) -> None:
         """Add a chromosome location to the location list.
 
         :param locations: NCBI map locations for a gene record.
-        :param location_list: A list to store chromosome locations.
         :param params: A transformed gene record
         """
         for i in range(len(locations)):
             loc = locations[i].strip()
-            location = dict()
+            location = {}
 
             if Annotation.ALT_LOC.value in loc:
                 loc = loc.split(f"{Annotation.ALT_LOC.value}")[0].strip()
@@ -419,26 +413,22 @@ class NCBI(Base):
                 # Location only gives chr
                 params["location_annotations"].append(loc)
 
-            # chr_location = self._get_chromosome_location(location, params)
-            # if chr_location:
-            #     location_list.append(chr_location)
-
-    def _set_centromere_location(self, loc: str, location: Dict) -> None:
+    def _set_centromere_location(self, loc: str, location: dict) -> None:
         """Set centromere location for a gene.
 
         :param loc: A gene location
         :param location: GA4GH location
         """
-        centromere_ix = re.search("cen", loc).start()  # type: ignore
+        centromere_ix = re.search("cen", loc).start()
         if "-" in loc:
             # Location gives both start and end
-            range_ix = re.search("-", loc).start()  # type: ignore
+            range_ix = re.search("-", loc).start()
             if "q" in loc:
                 location["chr"] = loc[:centromere_ix].strip()
                 location["start"] = "cen"
                 location["end"] = loc[range_ix + 1 :]
             elif "p" in loc:
-                p_ix = re.search("p", loc).start()  # type: ignore
+                p_ix = re.search("p", loc).start()
                 location["chr"] = loc[:p_ix].strip()
                 location["end"] = "cen"
                 location["start"] = loc[:range_ix]
@@ -464,7 +454,7 @@ class NCBI(Base):
 
         self._get_gene_gff(db, info_genes)
 
-        for gene in info_genes.keys():
+        for gene in info_genes:
             self._load_gene(info_genes[gene])
         _logger.info("Successfully transformed NCBI.")
 
@@ -482,9 +472,8 @@ class NCBI(Base):
                 self._assembly,
             ]
         ):
-            raise GeneNormalizerEtlError(
-                "Source metadata unavailable -- was data properly acquired before attempting to load DB?"
-            )
+            err_msg = "Source metadata unavailable -- was data properly acquired before attempting to load DB?"
+            raise GeneNormalizerEtlError(err_msg)
         metadata = SourceMeta(
             data_license="custom",
             data_license_url="https://www.ncbi.nlm.nih.gov/home/about/policies/",
@@ -495,11 +484,9 @@ class NCBI(Base):
                 "assembly_file": self._assembly_url,
             },
             rdp_url="https://reusabledata.org/ncbi-gene.html",
-            data_license_attributes={
-                "non_commercial": False,
-                "share_alike": False,
-                "attribution": False,
-            },
+            data_license_attributes=DataLicenseAttributes(
+                non_commercial=False, share_alike=False, attribution=False
+            ),
             genome_assemblies=[self._assembly],
         )
 
