@@ -1,14 +1,27 @@
 """Provides methods for handling queries."""
+
+import datetime
+import logging
 import re
-from datetime import datetime
-from typing import Any, Callable, Dict, List, Optional, Set, Tuple, TypeVar
+from collections.abc import Callable
+from typing import Any, TypeVar
 
-from ga4gh.core import core_models, ga4gh_identify
-from ga4gh.vrs import models
+from ga4gh.core import ga4gh_identify
+from ga4gh.core.models import (
+    Coding,
+    ConceptMapping,
+    Extension,
+    MappableConcept,
+    Relation,
+    code,
+)
+from ga4gh.vrs.models import SequenceLocation, SequenceReference
 
-from gene import ITEM_TYPES, NAMESPACE_LOOKUP, PREFIX_LOOKUP, logger
+from gene import ITEM_TYPES, NAMESPACE_LOOKUP, PREFIX_LOOKUP, __version__
 from gene.database import AbstractDatabase, DatabaseReadException
 from gene.schemas import (
+    NAMESPACE_TO_SYSTEM_URI,
+    SYSTEM_URI_TO_NAMESPACE,
     BaseGene,
     BaseNormalizationService,
     Gene,
@@ -26,7 +39,8 @@ from gene.schemas import (
     SourcePriority,
     UnmergedNormalizationService,
 )
-from gene.version import __version__
+
+_logger = logging.getLogger(__name__)
 
 NormService = TypeVar("NormService", bound=BaseNormalizationService)
 
@@ -58,7 +72,7 @@ class QueryHandler:
         self.db = database
 
     @staticmethod
-    def _emit_warnings(query_str: str) -> List:
+    def _emit_warnings(query_str: str) -> list:
         """Emit warnings if query contains non breaking space characters.
 
         :param query_str: query string
@@ -72,13 +86,13 @@ class QueryHandler:
                     "non_breaking_space_characters": "Query contains non-breaking space characters"
                 }
             ]
-            logger.warning(
-                f"Query ({query_str}) contains non-breaking space characters."
+            _logger.warning(
+                "Query (%s) contains non-breaking space characters.", query_str
             )
         return warnings
 
     @staticmethod
-    def _transform_sequence_location(loc: Dict) -> models.SequenceLocation:
+    def _transform_sequence_location(loc: dict) -> SequenceLocation:
         """Transform a sequence location to VRS sequence location
 
         :param loc: GeneSequenceLocation represented as a dict
@@ -86,73 +100,40 @@ class QueryHandler:
         """
         refget_ac = loc["sequence_id"].split("ga4gh:")[-1]
 
-        return models.SequenceLocation(
-            sequenceReference=models.SequenceReference(refgetAccession=refget_ac),
+        return SequenceLocation(
+            sequenceReference=SequenceReference(refgetAccession=refget_ac),
             start=int(loc["start"]),
             end=int(loc["end"]),
         )
 
-    # @staticmethod
-    # def _transform_chromosome_location(loc: Dict) -> ChromosomeLocation:
-    #     """Transform a chromosome location to VRS chromosome location
+    def _transform_location(self, loc: dict) -> dict:
+        """Transform a sequence location to VRS sequence location
 
-    #     :param loc: Chromosome location
-    #     :return: VRS chromosome location
-    #     """
-    #     return ChromosomeLocation(
-    #         species_id=loc["species_id"],
-    #         chr=loc["chr"],
-    #         start=loc["start"],
-    #         end=loc["end"]
-    #     )
-
-    def _transform_location(self, loc: Dict) -> Dict:
-        """Transform a sequence/chromosome location to VRS sequence/chromosome location
-
-        :param loc: Sequence or Chromosome location
-        :return: VRS sequence or chromosome location represented as a dictionary
+        :param loc: Sequence location
+        :return: VRS sequence location represented as a dictionary
         """
-        # if loc["type"] == "SequenceLocation":
-        #     transformed_loc = self._transform_sequence_location(loc)
-        # else:
-        #     transformed_loc = self._transform_chromosome_location(loc)
-        # Only support sequence locations atm
         transformed_loc = self._transform_sequence_location(loc)
         transformed_loc.id = ga4gh_identify(transformed_loc)
         return transformed_loc.model_dump(exclude_none=True)
 
-    def _transform_locations(self, record: Dict) -> Dict:
-        """Transform gene locations to VRS Chromosome/Sequence Locations
+    def _transform_locations(self, record: dict) -> dict:
+        """Transform gene locations to VRS Sequence Locations
 
         :param record: original record
         :return: record with transformed locations attributes, if applicable
         """
-        record_locations = list()
+        record_locations = []
         if "locations" in record:
-            for loc in record["locations"]:
-                if loc["type"] == "SequenceLocation":
-                    record_locations.append(self._transform_location(loc))
+            record_locations.extend(
+                self._transform_location(loc)
+                for loc in record["locations"]
+                if loc["type"] == "SequenceLocation"
+            )
         record["locations"] = record_locations
         return record
 
-    def _get_src_name(self, concept_id: str) -> SourceName:
-        """Get source name enum from ID.
-
-        :param concept_id: candidate concept ID string to check
-        :return: SourceName option
-        :raise: ValueError if unrecognized ID provided
-        """
-        if concept_id.startswith(NamespacePrefix.ENSEMBL.value):
-            return SourceName.ENSEMBL
-        elif concept_id.startswith(NamespacePrefix.NCBI.value):
-            return SourceName.NCBI
-        elif concept_id.startswith(NamespacePrefix.HGNC.value):
-            return SourceName.HGNC
-        else:
-            raise ValueError("Invalid or unrecognized concept ID provided")
-
     def _add_record(
-        self, response: Dict[str, Dict], item: Dict, match_type: MatchType
+        self, response: dict[str, dict], item: dict, match_type: MatchType
     ) -> None:
         """Add individual record (i.e. Item in DynamoDB) to response object
 
@@ -166,7 +147,7 @@ class QueryHandler:
         src_name = item["src_name"]
 
         matches = response["source_matches"]
-        if src_name not in matches.keys():
+        if src_name not in matches:
             pass
         elif matches[src_name] is None:
             matches[src_name] = {
@@ -177,7 +158,7 @@ class QueryHandler:
             matches[src_name]["records"].append(gene)
 
     def _fetch_record(
-        self, response: Dict[str, Dict], concept_id: str, match_type: MatchType
+        self, response: dict[str, dict], concept_id: str, match_type: MatchType
     ) -> None:
         """Add fetched record to response
 
@@ -188,18 +169,20 @@ class QueryHandler:
         try:
             match = self.db.get_record_by_id(concept_id, case_sensitive=False)
         except DatabaseReadException as e:
-            logger.error(
-                f"Encountered DatabaseReadException looking up {concept_id}: {e}"
+            _logger.error(
+                "Encountered DatabaseReadException looking up %s: %s", concept_id, e
             )
         else:
             if match:
                 self._add_record(response, match, match_type)
             else:
-                logger.error(
-                    f"Unable to find expected record for {concept_id} matching as {match_type}"
-                )  # noqa: E501
+                _logger.error(
+                    "Unable to find expected record for %s matching as %s",
+                    concept_id,
+                    match_type,
+                )
 
-    def _post_process_resp(self, resp: Dict) -> Dict:
+    def _post_process_resp(self, resp: dict) -> dict:
         """Fill all empty source_matches slots with NO_MATCH results and
         sort source records by descending `match_type`.
 
@@ -207,7 +190,7 @@ class QueryHandler:
         :return: response object with empty source slots filled with NO_MATCH results
             and corresponding source metadata
         """
-        for src_name in resp["source_matches"].keys():
+        for src_name in resp["source_matches"]:
             if resp["source_matches"][src_name] is None:
                 resp["source_matches"][src_name] = {
                     "match_type": MatchType.NO_MATCH,
@@ -220,7 +203,7 @@ class QueryHandler:
                     records = sorted(records, key=lambda k: k.match_type, reverse=True)
         return resp
 
-    def _get_search_response(self, query: str, sources: Set[str]) -> Dict:
+    def _get_search_response(self, query: str, sources: set[str]) -> dict:
         """Return response as dict where key is source name and value is a list of
         records.
 
@@ -237,18 +220,17 @@ class QueryHandler:
             return self._post_process_resp(resp)
         query_l = query.lower()
 
-        queries = list()
-        if [p for p in PREFIX_LOOKUP.keys() if query_l.startswith(p)]:
+        queries = []
+        if [p for p in PREFIX_LOOKUP if query_l.startswith(p)]:
             queries.append((query_l, RecordType.IDENTITY.value))
 
-        for prefix in [p for p in NAMESPACE_LOOKUP.keys() if query_l.startswith(p)]:
+        for prefix in [p for p in NAMESPACE_LOOKUP if query_l.startswith(p)]:
             term = f"{NAMESPACE_LOOKUP[prefix].lower()}:{query_l}"
             queries.append((term, RecordType.IDENTITY.value))
 
-        for match in ITEM_TYPES.values():
-            queries.append((query_l, match))
+        queries.extend((query_l, match) for match in ITEM_TYPES.values())
 
-        matched_concept_ids = list()
+        matched_concept_ids = []
         for term, item_type in queries:
             try:
                 if item_type == RecordType.IDENTITY.value:
@@ -263,9 +245,11 @@ class QueryHandler:
                             matched_concept_ids.append(ref)
 
             except DatabaseReadException as e:
-                logger.error(
-                    f"Encountered DatabaseReadException looking up {item_type}"
-                    f" {term}: {e}"
+                _logger.error(
+                    "Encountered DatabaseReadException looking up %s %s: ",
+                    item_type,
+                    term,
+                    e,
                 )
                 continue
 
@@ -278,14 +262,16 @@ class QueryHandler:
 
         :return: Service Meta
         """
-        return ServiceMeta(version=__version__, response_datetime=str(datetime.now()))
+        return ServiceMeta(
+            version=__version__,
+            response_datetime=str(datetime.datetime.now(tz=datetime.UTC)),
+        )
 
     def search(
         self,
         query_str: str,
         incl: str = "",
         excl: str = "",
-        **params,
     ) -> SearchService:
         """Return highest match for each source.
 
@@ -308,10 +294,9 @@ class QueryHandler:
         possible_sources = {
             name.value.lower(): name.value for name in SourceName.__members__.values()
         }
-        sources = dict()
-        for k, v in possible_sources.items():
-            if self.db.get_source_metadata(v):
-                sources[k] = v
+        sources = {
+            k: v for k, v in possible_sources.items() if self.db.get_source_metadata(v)
+        }
 
         if not incl and not excl:
             query_sources = set(sources.values())
@@ -323,7 +308,7 @@ class QueryHandler:
             invalid_sources = []
             query_sources = set()
             for source in req_sources:
-                if source.lower() in sources.keys():
+                if source.lower() in sources:
                     query_sources.add(sources[source.lower()])
                 else:
                     invalid_sources.append(source)
@@ -336,10 +321,10 @@ class QueryHandler:
             invalid_sources = []
             query_sources = set()
             for req_l, req in req_excl_dict.items():
-                if req_l not in sources.keys():
+                if req_l not in sources:
                     invalid_sources.append(req)
             for src_l, src in sources.items():
-                if src_l not in req_excl_dict.keys():
+                if src_l not in req_excl_dict:
                     query_sources.add(src)
             if invalid_sources:
                 detail = f"Invalid source name(s): {invalid_sources}"
@@ -360,25 +345,22 @@ class QueryHandler:
         """
         sources_meta = {}
         gene = response.gene
-        sources = [response.normalized_id.split(":")[0]]
-        if gene.mappings:
-            sources += [m.coding.system for m in gene.mappings]
+
+        sources = []
+        for m in gene.mappings or []:
+            ns = SYSTEM_URI_TO_NAMESPACE.get(m.coding.system)
+            if ns in PREFIX_LOOKUP:
+                sources.append(PREFIX_LOOKUP[ns])
 
         for src in sources:
-            try:
-                src_name = PREFIX_LOOKUP[src]
-            except KeyError:
-                # not an imported source
-                continue
-            else:
-                if src_name not in sources_meta:
-                    _source_meta = self.db.get_source_metadata(src_name)
-                    sources_meta[SourceName(src_name)] = SourceMeta(**_source_meta)
+            if src not in sources_meta:
+                _source_meta = self.db.get_source_metadata(src)
+                sources_meta[SourceName(src)] = SourceMeta(**_source_meta)
         response.source_meta_ = sources_meta
         return response
 
     def _add_alt_matches(
-        self, response: NormService, record: Dict, possible_concepts: List[str]
+        self, response: NormService, record: dict, possible_concepts: list[str]
     ) -> NormService:
         """Add alternate matches warning to response object
 
@@ -404,9 +386,9 @@ class QueryHandler:
     def _add_gene(
         self,
         response: NormalizeService,
-        record: Dict,
+        record: dict,
         match_type: MatchType,
-        possible_concepts: Optional[List[str]] = None,
+        possible_concepts: list[str] | None = None,
     ) -> NormalizeService:
         """Add core Gene object to response.
 
@@ -414,42 +396,70 @@ class QueryHandler:
         :param record: Gene record
         :param match_type: query's match type
         :param possible_concepts: List of other normalized concepts found
+        :raises ValueError: If source of record's concept ID or xrefs/associated with
+            sources is not a valid ``NamespacePrefix``
         :return: Response with core Gene
         """
-        gene_obj = core_models.Gene(
+
+        def _create_concept_mapping(
+            concept_id: str, relation: Relation = Relation.RELATED_MATCH
+        ) -> ConceptMapping:
+            """Create concept mapping for identifier
+
+            ``system`` will use source homepage or namespace prefix, in that order of \
+            preference, if available.
+
+            :param concept_id: A lowercase concept identifier represented as a curie
+            :param relation: SKOS mapping relationship, default is relatedMatch
+            :raises ValueError: If source of concept ID is not a valid
+                ``NamespacePrefix``
+            :return: Concept mapping for identifier
+            """
+            source, source_id = concept_id.split(":")
+
+            try:
+                source = NamespacePrefix(source)
+            except ValueError as e:
+                err_msg = f"Namespace prefix not supported: {source}"
+                raise ValueError(err_msg) from e
+
+            system = NAMESPACE_TO_SYSTEM_URI.get(source, source)
+            code_ = concept_id.upper() if source == NamespacePrefix.HGNC else source_id
+
+            return ConceptMapping(
+                coding=Coding(code=code(code_), system=system), relation=relation
+            )
+
+        gene_obj = MappableConcept(
             id=f"normalize.gene.{record['concept_id']}",
+            primaryCode=code(root=record["concept_id"]),
             label=record["symbol"],
+            conceptType="Gene",
         )
 
         # mappings
+        mappings = [
+            _create_concept_mapping(record["concept_id"], relation=Relation.EXACT_MATCH)
+        ]
         source_ids = record.get("xrefs", []) + record.get("associated_with", [])
-        mappings = []
-        for source_id in source_ids:
-            system, code = source_id.split(":")
-            mappings.append(
-                core_models.Mapping(
-                    coding=core_models.Coding(
-                        code=core_models.Code(code), system=system.lower()
-                    ),
-                    relation=core_models.Relation.RELATED_MATCH,
-                )
-            )
+        mappings.extend(_create_concept_mapping(source_id) for source_id in source_ids)
         if mappings:
             gene_obj.mappings = mappings
 
+        # extensions
+        extensions = []
+
         # aliases
         aliases = set()
         for key in ["previous_symbols", "aliases"]:
-            if key in record and record[key]:
+            if record.get(key):
                 val = record[key]
                 if isinstance(val, str):
                     val = [val]
                 aliases.update(val)
         if aliases:
-            gene_obj.aliases = list(aliases)
+            extensions.append(Extension(name="aliases", value=list(aliases)))
 
-        # extensions
-        extensions = []
         extension_and_record_labels = [
             ("symbol_status", "symbol_status"),
             ("approved_name", "label"),
@@ -458,10 +468,8 @@ class QueryHandler:
             ("strand", "strand"),
         ]
         for ext_label, record_label in extension_and_record_labels:
-            if record_label in record and record[record_label]:
-                extensions.append(
-                    core_models.Extension(name=ext_label, value=record[record_label])
-                )
+            if record.get(record_label):
+                extensions.append(Extension(name=ext_label, value=record[record_label]))
 
         record_locations = {}
         if record["item_type"] == RecordType.IDENTITY:
@@ -469,27 +477,26 @@ class QueryHandler:
             if locs:
                 record_locations[f"{record['src_name'].lower()}_locations"] = locs
         elif record["item_type"] == RecordType.MERGER:
-            for k, v in record.items():
-                if k.endswith("locations") and v:
-                    record_locations[k] = v
+            record_locations.update(
+                {k: v for k, v in record.items() if k.endswith("locations") and v}
+            )
 
         for loc_name, locations in record_locations.items():
-            transformed_locs = []
-            for loc in locations:
-                if loc["type"] == "SequenceLocation":
-                    transformed_locs.append(self._transform_location(loc))
+            transformed_locs = [
+                self._transform_location(loc)
+                for loc in locations
+                if loc["type"] == "SequenceLocation"
+            ]
 
             if transformed_locs:
-                extensions.append(
-                    core_models.Extension(name=loc_name, value=transformed_locs)
-                )
+                extensions.append(Extension(name=loc_name, value=transformed_locs))
 
         # handle gene types separately because they're wonky
         if record["item_type"] == RecordType.IDENTITY:
             gene_type = record.get("gene_type")
             if gene_type:
                 extensions.append(
-                    core_models.Extension(
+                    Extension(
                         name=GeneTypeFieldName[record["src_name"].upper()].value,
                         value=gene_type,
                     )
@@ -498,10 +505,9 @@ class QueryHandler:
             for f in GeneTypeFieldName:
                 field_name = f.value
                 values = record.get(field_name, [])
-                for value in values:
-                    extensions.append(
-                        core_models.Extension(name=field_name, value=value)
-                    )
+                extensions.extend(
+                    Extension(name=field_name, value=value) for value in values
+                )
         if extensions:
             gene_obj.extensions = extensions
 
@@ -509,14 +515,13 @@ class QueryHandler:
         if possible_concepts:
             response = self._add_alt_matches(response, record, possible_concepts)
 
-        response.normalized_id = record["concept_id"]
         response.gene = gene_obj
         response = self._add_merged_meta(response)
         response.match_type = match_type
         return response
 
     @staticmethod
-    def _record_order(record: Dict) -> Tuple[int, str]:
+    def _record_order(record: dict) -> tuple[int, str]:
         """Construct priority order for matching. Only called by sort().
 
         :param record: individual record item in iterable to sort
@@ -526,23 +531,7 @@ class QueryHandler:
         source_rank = SourcePriority[src]
         return source_rank, record["concept_id"]
 
-    @staticmethod
-    def _handle_failed_merge_ref(record: Dict, response: Dict, query: str) -> Dict:
-        """Log + fill out response for a failed merge reference lookup.
-
-        :param record: record containing failed merge_ref
-        :param response: in-progress response object
-        :param query: original query value
-        :return: response with no match
-        """
-        logger.error(
-            f"Merge ref lookup failed for ref {record['merge_ref']} "
-            f"in record {record['concept_id']} from query {query}"
-        )
-        response["match_type"] = MatchType.NO_MATCH
-        return response
-
-    def _prepare_normalized_response(self, query: str) -> Dict[str, Any]:
+    def _prepare_normalized_response(self, query: str) -> dict[str, Any]:
         """Provide base response object for normalize endpoints.
 
         :param query: user-provided query
@@ -553,7 +542,8 @@ class QueryHandler:
             "match_type": MatchType.NO_MATCH,
             "warnings": self._emit_warnings(query),
             "service_meta_": ServiceMeta(
-                version=__version__, response_datetime=str(datetime.now())
+                version=__version__,
+                response_datetime=str(datetime.datetime.now(tz=datetime.UTC)),
             ),
         }
 
@@ -566,9 +556,9 @@ class QueryHandler:
         >>> from gene.database import create_db
         >>> q = QueryHandler(create_db())
         >>> result = q.normalize("BRAF")
-        >>> result.normalized_id
+        >>> result.gene.primaryCode.root
         'hgnc:1097'
-        >>> result.aliases
+        >>> next(ext for ext in result.gene.extensions if ext.name == "aliases").value
         ['BRAF1', 'RAFB1', 'B-raf', 'NS7', 'B-RAF1']
 
         :param query: String to find normalized concept for
@@ -580,10 +570,10 @@ class QueryHandler:
     def _resolve_merge(
         self,
         response: NormService,
-        record: Dict,
+        record: dict,
         match_type: MatchType,
         callback: Callable,
-        possible_concepts: Optional[List[str]] = None,
+        possible_concepts: list[str] | None = None,
     ) -> NormService:
         """Given a record, return the corresponding normalized record
 
@@ -600,16 +590,18 @@ class QueryHandler:
             merge = self.db.get_record_by_id(merge_ref, False, True)
             if merge is None:
                 query = response.query
-                logger.error(
-                    f"Merge ref lookup failed for ref {record['merge_ref']} "
-                    f"in record {record['concept_id']} from query `{query}`"
+                _logger.error(
+                    "Merge ref lookup failed for ref %s in record %s from query `%s`",
+                    record["merge_ref"],
+                    record["concept_id"],
+                    query,
                 )
                 return response
-            else:
-                return callback(response, merge, match_type, possible_concepts)
-        else:
-            # record is sole member of concept group
-            return callback(response, record, match_type, possible_concepts)
+
+            return callback(response, merge, match_type, possible_concepts)
+
+        # record is sole member of concept group
+        return callback(response, record, match_type, possible_concepts)
 
     def _perform_normalized_lookup(
         self, response: NormService, query: str, response_builder: Callable
@@ -619,6 +611,7 @@ class QueryHandler:
         :param response: in-progress response object
         :param query: user-provided query
         :param response_builder: response constructor callback method
+        :raises ValueError: If a matching record is null
         :return: completed service response object
         """
         if query == "":
@@ -643,16 +636,15 @@ class QueryHandler:
             matching_records = [
                 self.db.get_record_by_id(ref, False) for ref in matching_refs
             ]
-            matching_records.sort(key=self._record_order)  # type: ignore
+            matching_records.sort(key=self._record_order)
 
-            if len(matching_refs) > 1:
-                possible_concepts = [ref for ref in matching_refs]
-            else:
-                possible_concepts = None
+            possible_concepts = list(matching_refs) if len(matching_refs) > 1 else None
 
             # attempt merge ref resolution until successful
             for match in matching_records:
-                assert match is not None
+                if match is None:
+                    err_msg = "Matching record must be nonnull"
+                    raise ValueError(err_msg)
                 record = self.db.get_record_by_id(match["concept_id"], False)
                 if record:
                     match_type_value = MatchType[match_type.value.upper()]
@@ -668,9 +660,9 @@ class QueryHandler:
     def _add_normalized_records(
         self,
         response: UnmergedNormalizationService,
-        normalized_record: Dict,
+        normalized_record: dict,
         match_type: MatchType,
-        possible_concepts: Optional[List[str]] = None,
+        possible_concepts: list[str] | None = None,
     ) -> UnmergedNormalizationService:
         """Add individual records to unmerged normalize response.
 
@@ -688,12 +680,11 @@ class QueryHandler:
             meta = self.db.get_source_metadata(record_source.value)
             response.source_matches[record_source] = MatchesNormalized(
                 records=[BaseGene(**self._transform_locations(normalized_record))],
-                source_meta_=meta,  # type: ignore
+                source_meta_=meta,
             )
         else:
-            concept_ids = [normalized_record["concept_id"]] + normalized_record.get(
-                "xrefs", []
-            )
+            xrefs = normalized_record.get("xrefs") or []
+            concept_ids = [normalized_record["concept_id"], *xrefs]
             for concept_id in concept_ids:
                 record = self.db.get_record_by_id(concept_id, case_sensitive=False)
                 if not record:
@@ -705,8 +696,7 @@ class QueryHandler:
                 else:
                     meta = self.db.get_source_metadata(record_source.value)
                     response.source_matches[record_source] = MatchesNormalized(
-                        records=[gene],
-                        source_meta_=meta,  # type: ignore
+                        records=[gene], source_meta_=meta
                     )
         if possible_concepts:
             response = self._add_alt_matches(
