@@ -1,21 +1,24 @@
 """Contains data models for representing VICC normalized gene records."""
-from enum import Enum, IntEnum
-from typing import Dict, List, Literal, Optional, Union
 
-from ga4gh.core import core_models
-from ga4gh.vrs import models
+from enum import Enum, IntEnum
+from typing import Annotated, Literal
+
+from ga4gh.core.models import (
+    MappableConcept,
+)
+from ga4gh.vrs.models import SequenceLocation
 from pydantic import (
     BaseModel,
     ConfigDict,
     StrictBool,
     StrictInt,
     StrictStr,
-    constr,
+    StringConstraints,
 )
 
-from gene.version import __version__
+from gene import __version__
 
-CURIE = constr(pattern=r"^\w[^:]*:.+$")
+CURIE_REGEX = r"^\w[^:]*:.+$"
 
 
 class SymbolStatus(str, Enum):
@@ -59,7 +62,6 @@ class MatchType(IntEnum):
     ALIAS = 60
     XREF = 60
     ASSOCIATED_WITH = 60
-    FUZZY_MATCH = 20
     NO_MATCH = 0
 
 
@@ -69,17 +71,9 @@ class GeneSequenceLocation(BaseModel):
     type: Literal["SequenceLocation"] = "SequenceLocation"
     start: StrictInt
     end: StrictInt
-    sequence_id: constr(pattern=r"^ga4gh:SQ.[0-9A-Za-z_\-]{32}$")  # noqa: F722
-
-
-# class GeneChromosomeLocation(BaseModel):
-#     """Chromosome Location model when storing in DynamDB."""
-
-#     type: Literal["ChromosomeLocation"] = "ChromosomeLocation"
-#     species_id: Literal["taxonomy:9606"] = "taxonomy:9606"
-#     chr: StrictStr
-#     start: StrictStr
-#     end: StrictStr
+    sequence_id: Annotated[
+        str, StringConstraints(pattern=r"^ga4gh:SQ.[0-9A-Za-z_\-]{32}$")
+    ]
 
 
 class BaseGene(BaseModel):
@@ -87,22 +81,18 @@ class BaseGene(BaseModel):
     /search and /normalize_unmerged.
     """
 
-    concept_id: CURIE
+    concept_id: Annotated[str, StringConstraints(pattern=CURIE_REGEX)]
     symbol: StrictStr
-    symbol_status: Optional[SymbolStatus] = None
-    label: Optional[StrictStr] = None
-    strand: Optional[Strand] = None
-    location_annotations: List[StrictStr] = []
-    locations: Union[
-        List[models.SequenceLocation], List[GeneSequenceLocation]
-        # List[Union[SequenceLocation, ChromosomeLocation]],
-        # List[Union[GeneSequenceLocation, GeneChromosomeLocation]]  # dynamodb
-    ] = []
-    aliases: List[StrictStr] = []
-    previous_symbols: List[StrictStr] = []
-    xrefs: List[CURIE] = []
-    associated_with: List[CURIE] = []
-    gene_type: Optional[StrictStr] = None
+    symbol_status: SymbolStatus | None = None
+    label: StrictStr | None = None
+    strand: Strand | None = None
+    location_annotations: list[StrictStr] = []
+    locations: list[SequenceLocation] | list[GeneSequenceLocation] = []
+    aliases: list[StrictStr] = []
+    previous_symbols: list[StrictStr] = []
+    xrefs: list[Annotated[str, StringConstraints(pattern=CURIE_REGEX)]] = []
+    associated_with: list[Annotated[str, StringConstraints(pattern=CURIE_REGEX)]] = []
+    gene_type: StrictStr | None = None
 
 
 class Gene(BaseGene):
@@ -131,14 +121,6 @@ class Gene(BaseGene):
     )
 
 
-class GeneGroup(Gene):
-    """A grouping of genes based on common attributes."""
-
-    description: StrictStr
-    type_identifier: StrictStr
-    genes: List[Gene] = []
-
-
 class SourceName(Enum):
     """Define string constraints to ensure consistent capitalization."""
 
@@ -160,7 +142,7 @@ class SourceIDAfterNamespace(Enum):
 
     HGNC = ""
     ENSEMBL = "ENSG"
-    NCBI = ""
+    NCBI = ""  # noqa: PIE796
 
 
 class NamespacePrefix(Enum):
@@ -196,6 +178,34 @@ class NamespacePrefix(Enum):
     RFAM = "rfam"
 
 
+# Source to URI. Will use source homepage
+NAMESPACE_TO_SYSTEM_URI: dict[NamespacePrefix, str] = {
+    NamespacePrefix.HGNC: "https://www.genenames.org",
+    NamespacePrefix.ENSEMBL: "https://www.ensembl.org",
+    NamespacePrefix.NCBI: "https://www.ncbi.nlm.nih.gov/gene/",
+    NamespacePrefix.ENTREZ: "https://www.ncbi.nlm.nih.gov/gene/",
+    NamespacePrefix.VEGA: "https://www.sanger.ac.uk/tool/vega-genome-browser/",
+    NamespacePrefix.UCSC: "https://genome.ucsc.edu",
+    NamespacePrefix.ENA: "https://www.ebi.ac.uk/ena/",
+    NamespacePrefix.REFSEQ: "https://www.ncbi.nlm.nih.gov/refseq/",
+    NamespacePrefix.CCDS: "https://www.ncbi.nlm.nih.gov/projects/CCDS/CcdsBrowse.cgi",
+    NamespacePrefix.UNIPROT: "https://www.uniprot.org",
+    NamespacePrefix.PUBMED: "https://pubmed.ncbi.nlm.nih.gov",
+    NamespacePrefix.COSMIC: "https://cancer.sanger.ac.uk/cosmic/",
+    NamespacePrefix.OMIM: "https://www.omim.org",
+    NamespacePrefix.SNORNABASE: "https://www-snorna.biotoul.fr",
+    NamespacePrefix.PSEUDOGENE: "http://pseudogene.org",
+    NamespacePrefix.MEROPS: "https://www.ebi.ac.uk/merops/",
+    NamespacePrefix.IUPHAR: "https://www.guidetopharmacology.org",
+    NamespacePrefix.RFAM: "https://rfam.org",
+}
+
+# URI to source
+SYSTEM_URI_TO_NAMESPACE = {
+    system_uri: ns.value for ns, system_uri in NAMESPACE_TO_SYSTEM_URI.items()
+}
+
+
 class DataLicenseAttributes(BaseModel):
     """Define constraints for data license attributes."""
 
@@ -228,10 +238,10 @@ class SourceMeta(BaseModel):
     data_license: StrictStr
     data_license_url: StrictStr
     version: StrictStr
-    data_url: Dict[StrictStr, StrictStr]  # TODO strictness necessary?
-    rdp_url: Optional[StrictStr] = None
-    data_license_attributes: Dict[StrictStr, StrictBool]
-    genome_assemblies: List[StrictStr] = []
+    data_url: dict[StrictStr, StrictStr]  # TODO strictness necessary?
+    rdp_url: StrictStr | None = None
+    data_license_attributes: DataLicenseAttributes
+    genome_assemblies: list[StrictStr] = []
 
     model_config = ConfigDict(
         json_schema_extra={
@@ -259,7 +269,7 @@ class SourceMeta(BaseModel):
 class SourceSearchMatches(BaseModel):
     """Container for matching information from an individual source."""
 
-    records: List[Gene] = []
+    records: list[Gene] = []
     source_meta_: SourceMeta
 
     model_config = ConfigDict(json_schema_extra={"example": {}})  # TODO
@@ -271,9 +281,9 @@ class ServiceMeta(BaseModel):
     name: Literal["gene-normalizer"] = "gene-normalizer"
     version: StrictStr
     response_datetime: StrictStr
-    url: Literal[
+    url: Literal["https://github.com/cancervariants/gene-normalization"] = (
         "https://github.com/cancervariants/gene-normalization"
-    ] = "https://github.com/cancervariants/gene-normalization"  # noqa: E501
+    )
 
     model_config = ConfigDict(
         json_schema_extra={
@@ -291,8 +301,8 @@ class SearchService(BaseModel):
     """Define model for returning highest match typed concepts from sources."""
 
     query: StrictStr
-    warnings: List[Dict] = []
-    source_matches: Dict[SourceName, SourceSearchMatches]
+    warnings: list[dict] = []
+    source_matches: dict[SourceName, SourceSearchMatches]
     service_meta_: ServiceMeta
 
     model_config = ConfigDict(json_schema_extra={})  # TODO
@@ -312,7 +322,7 @@ class BaseNormalizationService(BaseModel):
     """Base method providing shared attributes to Normalization service classes."""
 
     query: StrictStr
-    warnings: List[Dict] = []
+    warnings: list[dict] = []
     match_type: MatchType
     service_meta_: ServiceMeta
 
@@ -320,9 +330,8 @@ class BaseNormalizationService(BaseModel):
 class NormalizeService(BaseNormalizationService):
     """Define model for returning normalized concept."""
 
-    normalized_id: Optional[str] = None
-    gene: Optional[core_models.Gene] = None
-    source_meta_: Dict[SourceName, SourceMeta] = {}
+    gene: MappableConcept | None = None
+    source_meta_: dict[SourceName, SourceMeta] = {}
 
     model_config = ConfigDict(
         json_schema_extra={
@@ -330,26 +339,38 @@ class NormalizeService(BaseNormalizationService):
                 "query": "BRAF",
                 "warnings": [],
                 "match_type": 100,
-                "normalized_id": "hgnc:1037",
                 "gene": {
-                    "type": "Gene",
+                    "conceptType": "Gene",
                     "id": "normalize.gene.hgnc:1097",
+                    "primaryCode": "hgnc:1097",
                     "label": "BRAF",
                     "mappings": [
                         {
-                            "coding": {"code": "673", "system": "ncbigene"},
+                            "coding": {
+                                "code": "HGNC:1097",
+                                "system": "https://www.genenames.org",
+                            },
+                            "relation": "exactMatch",
+                        },
+                        {
+                            "coding": {
+                                "code": "673",
+                                "system": "https://www.ncbi.nlm.nih.gov/gene/",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "ENSG00000157764", "system": "ensembl"},
+                            "coding": {
+                                "code": "ENSG00000157764",
+                                "system": "https://www.ensembl.org",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "CCDS5863", "system": "ccds"},
-                            "relation": "relatedMatch",
-                        },
-                        {
-                            "coding": {"code": "1943", "system": "iuphar"},
+                            "coding": {
+                                "code": "1943",
+                                "system": "https://www.guidetopharmacology.org",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
@@ -357,35 +378,52 @@ class NormalizeService(BaseNormalizationService):
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "BRAF", "system": "cosmic"},
+                            "coding": {
+                                "code": "BRAF",
+                                "system": "https://cancer.sanger.ac.uk/cosmic",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "2284096", "system": "pubmed"},
+                            "coding": {
+                                "code": "2284096",
+                                "system": "https://pubmed.ncbi.nlm.nih.gov",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "uc003vwc.5", "system": "ucsc"},
+                            "coding": {
+                                "code": "uc003vwc.5",
+                                "system": "https://genome.ucsc.edu",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "164757", "system": "omim"},
+                            "coding": {
+                                "code": "164757",
+                                "system": "https://www.omim.org",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "NM_004333", "system": "refseq"},
+                            "coding": {
+                                "code": "NM_004333",
+                                "system": "https://www.ncbi.nlm.nih.gov/refseq/",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "CCDS87555", "system": "ccds"},
+                            "coding": {
+                                "code": "P15056",
+                                "system": "https://www.uniprot.org",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "P15056", "system": "uniprot"},
-                            "relation": "relatedMatch",
-                        },
-                        {
-                            "coding": {"code": "M95712", "system": "ena.embl"},
+                            "coding": {
+                                "code": "M95712",
+                                "system": "https://www.ebi.ac.uk/ena/",
+                            },
                             "relation": "relatedMatch",
                         },
                         {
@@ -393,34 +431,65 @@ class NormalizeService(BaseNormalizationService):
                             "relation": "relatedMatch",
                         },
                         {
-                            "coding": {"code": "1565476", "system": "pubmed"},
+                            "coding": {
+                                "code": "1565476",
+                                "system": "https://pubmed.ncbi.nlm.nih.gov",
+                            },
                             "relation": "relatedMatch",
                         },
                     ],
-                    "aliases": ["BRAF1", "RAFB1", "B-raf", "NS7", "B-RAF1"],
                     "extensions": [
+                        {
+                            "name": "aliases",
+                            "value": [
+                                "BRAF1",
+                                "BRAF-1",
+                                "RAFB1",
+                                "NS7",
+                                "B-RAF1",
+                                "B-raf",
+                            ],
+                        },
                         {
                             "name": "approved_name",
                             "value": "B-Raf proto-oncogene, serine/threonine kinase",
-                            "type": "Extension",
                         },
                         {
-                            "name": "symbol_status",
-                            "value": "approved",
-                            "type": "Extension",
+                            "name": "ensembl_locations",
+                            "value": [
+                                {
+                                    "type": "SequenceLocation",
+                                    "sequenceReference": {
+                                        "type": "SequenceReference",
+                                        "refgetAccession": "SQ.F-LrLMe1SRpfUZHkQmvkVKFEGaoDeHul",
+                                    },
+                                    "start": 140719326,
+                                    "end": 140924929,
+                                }
+                            ],
                         },
-                        # {
-                        #     "name": "chromosome_location",
-                        #     "value": {
-                        #         "id": "ga4gh:CL.O6yCQ1cnThOrTfK9YUgMlTfM6HTqbrKw",  # noqa: E501
-                        #         "type": "ChromosomeLocation",
-                        #         "species_id": "taxonomy:9606",
-                        #         "chr": "7",
-                        #         "end": "q34",
-                        #         "start": "q34",
-                        #     },
-                        #     "type": "Extension"
-                        # }
+                        {
+                            "name": "ncbi_locations",
+                            "value": [
+                                {
+                                    "type": "SequenceLocation",
+                                    "sequenceReference": {
+                                        "type": "SequenceReference",
+                                        "refgetAccession": "SQ.F-LrLMe1SRpfUZHkQmvkVKFEGaoDeHul",
+                                    },
+                                    "start": 140713327,
+                                    "end": 140924929,
+                                }
+                            ],
+                        },
+                        {"name": "ncbi_gene_type", "value": "protein-coding"},
+                        {
+                            "name": "hgnc_locus_type",
+                            "value": "gene with protein product",
+                        },
+                        {"name": "ensembl_biotype", "value": "protein_coding"},
+                        {"name": "strand", "value": "-"},
+                        {"name": "symbol_status", "value": "approved"},
                     ],
                 },
                 "source_meta_": {
@@ -441,7 +510,7 @@ class NormalizeService(BaseNormalizationService):
                     },
                     "Ensembl": {
                         "data_license": "custom",
-                        "data_license_url": "https://useast.ensembl.org/info/about/legal/disclaimer.html",  # noqa: E501
+                        "data_license_url": "https://useast.ensembl.org/info/about/legal/disclaimer.html",
                         "version": "104",
                         "data_url": {
                             "genome_annotations": "ftp://ftp.ensembl.org/pub/current_gff3/homo_sapiens/Homo_sapiens.GRCh38.110.gff3.gz"
@@ -456,7 +525,7 @@ class NormalizeService(BaseNormalizationService):
                     },
                     "NCBI": {
                         "data_license": "custom",
-                        "data_license_url": "https://www.ncbi.nlm.nih.gov/home/about/policies/",  # noqa: E501
+                        "data_license_url": "https://www.ncbi.nlm.nih.gov/home/about/policies/",
                         "version": "20210813",
                         "data_url": {
                             "info_file": "ftp.ncbi.nlm.nih.govgene/DATA/GENE_INFO/Mammalia/Homo_sapiens.gene_info.gz",
@@ -486,7 +555,7 @@ class NormalizeService(BaseNormalizationService):
 class MatchesNormalized(BaseModel):
     """Matches associated with normalized concept from a single source."""
 
-    records: List[BaseGene] = []
+    records: list[BaseGene] = []
     source_meta_: SourceMeta
 
 
@@ -496,8 +565,10 @@ class UnmergedNormalizationService(BaseNormalizationService):
     attributes.
     """
 
-    normalized_concept_id: Optional[CURIE] = None
-    source_matches: Dict[SourceName, MatchesNormalized]
+    normalized_concept_id: (
+        Annotated[str, StringConstraints(pattern=CURIE_REGEX)] | None
+    ) = None
+    source_matches: dict[SourceName, MatchesNormalized]
 
     model_config = ConfigDict(
         json_schema_extra={
@@ -519,19 +590,10 @@ class UnmergedNormalizationService(BaseNormalizationService):
                                 "concept_id": "hgnc:108",
                                 "symbol": "ACHE",
                                 "symbol_status": "approved",
-                                "label": "acetylcholinesterase (Cartwright blood group)",  # noqa: E501
+                                "label": "acetylcholinesterase (Cartwright blood group)",
                                 "strand": None,
                                 "location_annotations": [],
-                                "locations": [
-                                    # {
-                                    #     "type": "ChromosomeLocation",
-                                    #     "id": "ga4gh:CL.VtdU_0lYXL_o95lXRUfhv-NDJVVpmKoD",  # noqa: E501
-                                    #     "species_id": "taxonomy:9606",
-                                    #     "chr": "7",
-                                    #     "start": "q22.1",
-                                    #     "end": "q22.1"
-                                    # }
-                                ],
+                                "locations": [],
                                 "aliases": ["3.1.1.7"],
                                 "previous_symbols": ["YT"],
                                 "xrefs": ["ncbigene:43", "ensembl:ENSG00000087085"],
@@ -573,16 +635,17 @@ class UnmergedNormalizationService(BaseNormalizationService):
                                 "concept_id": "ensembl:ENSG00000087085",
                                 "symbol": "ACHE",
                                 "symbol_status": None,
-                                "label": "acetylcholinesterase (Cartwright blood group)",  # noqa: E501
+                                "label": "acetylcholinesterase (Cartwright blood group)",
                                 "strand": "-",
                                 "location_annotations": [],
                                 "locations": [
                                     {
-                                        "id": "ga4gh:SL.dnydHb2Bnv5pwXjI4MpJmrZUADf5QLe1",  # noqa: E501
+                                        "id": "ga4gh:SL.4taOKYezIxUvFozs6c6OC0bJAQ2zwjxu",
+                                        "digest": "4taOKYezIxUvFozs6c6OC0bJAQ2zwjxu",
                                         "type": "SequenceLocation",
                                         "sequenceReference": {
                                             "type": "SequenceReference",
-                                            "refgetAccession": "SQ.F-LrLMe1SRpfUZHkQmvkVKFEGaoDeHul",  # noqa: E501
+                                            "refgetAccession": "SQ.F-LrLMe1SRpfUZHkQmvkVKFEGaoDeHul",
                                         },
                                         "start": 100889993,
                                         "end": 100896974,
@@ -597,7 +660,7 @@ class UnmergedNormalizationService(BaseNormalizationService):
                         ],
                         "source_meta_": {
                             "data_license": "custom",
-                            "data_license_url": "https://useast.ensembl.org/info/about/legal/disclaimer.html",  # noqa: E501
+                            "data_license_url": "https://useast.ensembl.org/info/about/legal/disclaimer.html",
                             "version": "104",
                             "data_url": {
                                 "genome_annotations": "ftp://ftp.ensembl.org/pub/current_gff3/homo_sapiens/Homo_sapiens.GRCh38.110.gff3.gz"
@@ -617,24 +680,17 @@ class UnmergedNormalizationService(BaseNormalizationService):
                                 "concept_id": "ncbigene:43",
                                 "symbol": "ACHE",
                                 "symbol_status": None,
-                                "label": "acetylcholinesterase (Cartwright blood group)",  # noqa: E501
+                                "label": "acetylcholinesterase (Cartwright blood group)",
                                 "strand": "-",
                                 "location_annotations": [],
                                 "locations": [
                                     {
-                                        # "type": "ChromosomeLocation",
-                                        # "id": "ga4gh:CL.VtdU_0lYXL_o95lXRUfhv-NDJVVpmKoD",  # noqa: E501
-                                        # "species_id": "taxonomy:9606",
-                                        # "chr": "7",
-                                        # "start": "q22.1",
-                                        # "end": "q22.1"
-                                    },
-                                    {
-                                        "id": "ga4gh:SL.U7vPSlX8eyCKdFSiROIsc9om0Y7pCm2g",  # noqa: E501
+                                        "id": "ga4gh:SL.OWr9DoyBhr2zpf4uLLcZSvsTSIDElU6R",
+                                        "digest": "OWr9DoyBhr2zpf4uLLcZSvsTSIDElU6R",
                                         "type": "SequenceLocation",
                                         "sequenceReference": {
                                             "type": "SequenceReference",
-                                            "refgetAccession": "SQ.F-LrLMe1SRpfUZHkQmvkVKFEGaoDeHul",  # noqa: E501
+                                            "refgetAccession": "SQ.F-LrLMe1SRpfUZHkQmvkVKFEGaoDeHul",
                                         },
                                         "start": 100889993,
                                         "end": 100896994,
@@ -649,7 +705,7 @@ class UnmergedNormalizationService(BaseNormalizationService):
                         ],
                         "source_meta_": {
                             "data_license": "custom",
-                            "data_license_url": "https://www.ncbi.nlm.nih.gov/home/about/policies/",  # noqa: E501
+                            "data_license_url": "https://www.ncbi.nlm.nih.gov/home/about/policies/",
                             "version": "20220407",
                             "data_url": {
                                 "info_file": "ftp.ncbi.nlm.nih.govgene/DATA/GENE_INFO/Mammalia/Homo_sapiens.gene_info.gz",
